/*
 *  Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 *  Licensed under the Eclipse Public License - v 1.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *           http://www.eclipse.org/legal/epl-v10.html
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package nl.cloudfarming.client.fleet.machine;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import javax.swing.Icon;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.fleet.model.MachineManager;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import nl.cloudfarming.client.model.EntitySavable;
import org.netbeans.api.actions.Savable;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

/**
 *
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
public class MachineDataObject extends MultiDataObject implements PropertyChangeListener {

    private final InstanceContent ic = new InstanceContent();
    private final Lookup lkp = new AbstractLookup(ic);
    private final Machine machine;
    private final MachineManager machineManager;
    private final static String ICON = "/nl/cloudfarming/client/fleet/machine/fleet.png";

    public MachineDataObject(FileObject fo, MultiFileLoader loader, MachineManager machineManager) throws DataObjectExistsException {
        super(fo, loader);
        this.machineManager = machineManager;
        this.machine = machineManager.load(fo);
        ic.add(machine);
        ic.add(machine.getGeometrical());
        ic.add(new SimpleLayer(machine.getName()));
        machine.addPropertyChangeListener(this);
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), lkp);
    }
    private static final Icon IMAGE_ICON = ImageUtilities.loadImageIcon(ICON, true);

    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        if (lkp.lookup(Savable.class) == null) {
            this.ic.add(new MachineSavable());
        }
    }

    private class MachineSavable extends EntitySavable {

        public MachineSavable() {
            super(machine, IMAGE_ICON);
            register();
        }

        @Override
        public void handleSave() throws IOException {
            dataObject().machineManager.save(machine);
            destroy();
        }

        /**
         * Cancel the save operation Removes this Savable from the lookup and
         * unregisters
         */
        @Override
        public void destroy() {
            dataObject().ic.remove(MachineSavable.this);
            unregister();
        }

        @Override
        protected String findDisplayName() {
            return machine.getName();
        }

        MachineDataObject dataObject() {
            return MachineDataObject.this;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof MachineSavable) {
                return dataObject() == ((MachineSavable) obj).dataObject();
            }
            return false;
        }

        @Override
        public int hashCode() {
            return dataObject().hashCode();
        }
    }
}
