package nl.cloudfarming.client.fleet.machine;

/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
import java.util.*;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.fleet.model.MachineProperty;

/**
 * This class defines the functionality for returning a list of machine models.
 * These models are added to the selection fields in the machine creation
 * wizard.
 *
 * @author Wytse Visser
 */
public interface MachineDataService {
    
    public List<Model> getModels();

    public class Model {

        private String type;
        private String brand;
        private String modelNumber;
        private Map<String, Object> properties = new HashMap<>();
        private List<StepInfo> additionalSteps;

        public Model(String type, String brand, String modelNumber) {
            this.type = type;
            this.brand = brand;
            this.modelNumber = modelNumber;
            additionalSteps = new ArrayList<>();
        }

        public String getBrand() {
            return brand;
        }

        public void setBrand(String brand) {
            this.brand = brand;
        }

        public String getType() {
            return type;
        }

        public void setType(String type) {
            this.type = type;
        }

        public String getModelNumber() {
            return modelNumber;
        }

        public void setModelNumber(String modelNumber) {
            this.modelNumber = modelNumber;
        }

        @Override
        public boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            final Model other = (Model) obj;
            if (!Objects.equals(this.type, other.type)) {
                return false;
            }
            if (!Objects.equals(this.brand, other.brand)) {
                return false;
            }
            if (!Objects.equals(this.modelNumber, other.modelNumber)) {
                return false;
            }
            return true;
        }

        @Override
        public int hashCode() {
            int hash = 3;
            hash = 89 * hash + Objects.hashCode(this.type);
            hash = 89 * hash + Objects.hashCode(this.brand);
            hash = 89 * hash + Objects.hashCode(this.modelNumber);
            return hash;
        }

        public void putProperty(String key, Object value) {
            properties.put(key, value);
        }

        public Object getProperty(String key) {
            return properties.get(key);
        }

        public void addAdditionalStep(StepInfo step) {
            additionalSteps.add(step);
        }

        public List<StepInfo> getAdditionalSteps() {
            return additionalSteps;
        }
    }

    public class StepInfo {

        private String name;
        private List<MachineProperty> properties = new ArrayList<>();

        public StepInfo(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }

        public void addProperty(MachineProperty property) {
            properties.add(property);
        }

        public List<MachineProperty> getProperties() {
            return properties;
        }
    }
}
