/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import nl.cloudfarming.client.fleet.machine.MachineDataService;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.fleet.model.MachineProperty;
import org.openide.WizardDescriptor;

/**
 *
 * @author Wytse Visser
 */
public final class MachineWizardOverviewPanel extends MachineWizardPanel {

    private MachineVisualOverviewPanel visualPanel;
    private Set<PropertyEntry> propertyNames;

    @Override
    protected MachineVisualOverviewPanel getVisual() {
        if (visualPanel == null) {
            visualPanel = new MachineVisualOverviewPanel();
        }
        return visualPanel;
    }

    @Override
    public void readSettings(WizardDescriptor wiz) {
        getVisual().removeAll();
        setTitle(((Machine)wiz.getProperty(MachineWizardIterator.PROP_MACHINE)).getName());

        getVisual().addProperty("Name", ((Machine)wiz.getProperty(MachineWizardIterator.PROP_MACHINE)).getName());
        MachineDataService.Model model = (MachineDataService.Model) wiz.getProperty(MachineWizardIterator.PROP_MODEL);
        if (model != null) {
            getVisual().addProperty("Type", model.getType());
            getVisual().addProperty("Brand", model.getBrand());
            getVisual().addProperty("Model number", model.getModelNumber());
        }
        for (PropertyEntry entry : getPropertyNames()) {
            String humanReadableName = entry.getHumanReadableName();
            Object value = wiz.getProperty(entry.getName());
            getVisual().addProperty(humanReadableName, value);
        }
        addDynamicProperties(wiz);
    }

    public void addProperty(String name, String humanReadableName) {
        getPropertyNames().add(new PropertyEntry(name, humanReadableName));
    }

    private void addDynamicProperties(WizardDescriptor wiz) {
        Iterator<List<MachineProperty>> iterator = MachineDynamicWizardPanel.getDynamicPropertyMap(wiz).values().iterator();
        while (iterator.hasNext()) {
            List<MachineProperty> dynamicProperties = iterator.next();
            for (MachineProperty property : dynamicProperties) {
                getVisual().addProperty(property.getProperty()); //property.getName(), property.getValue());
            }
        }
    }

    @Override
    public void storeSettings(WizardDescriptor wiz) {
        // Can't change anything on this panel, nothing to store
    }

    private Set<PropertyEntry> getPropertyNames() {
        if (propertyNames == null) {
            propertyNames = new HashSet<>();
        }
        return propertyNames;
    }

    private class PropertyEntry {

        private String name;
        private String humanReadableName;

        public PropertyEntry(String name, String humanReadableName) {
            this.name = name;
            this.humanReadableName = humanReadableName;
        }

        String getName() {
            return name;
        }

        String getHumanReadableName() {
            return humanReadableName;
        }
    }
}
