/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.machine.wizard;

import java.awt.Component;
import java.util.HashSet;
import java.util.Set;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.fleet.machine.windows.MachineOverviewController;
import nl.cloudfarming.client.fleet.machine.windows.MachineOverviewPanel;
import nl.cloudfarming.client.fleet.model.Machine;
import nl.cloudfarming.client.fleet.model.MachineProvider;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.netbeans.api.project.Project;
import org.openide.WizardDescriptor;
import org.openide.WizardValidationException;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * The panel for the initial step in the machine creation wizard.
 *
 * @author Wytse Visser
 */
@NbBundle.Messages({"MachineWizardStartPanel general properties step name=General details",
                    "MachineWizardStartPanel invalid input message=There are some errors please correct them"})
public class MachineWizardStartPanel extends MachineOverviewController<MachineOverviewPanel> implements WizardDescriptor.ValidatingPanel<WizardDescriptor>, WizardDescriptor.FinishablePanel<WizardDescriptor> {

    private final Set<ChangeListener> listeners = new HashSet<>(1);
    private MachineWizardFactory factory;
    private Machine machineDataHolder;
    private ASBodyPanel overviewDecorator;

    public MachineWizardStartPanel(MachineWizardFactory factory, Machine machineDataHolder) {
        super(new MachineOverviewPanel(), machineDataHolder, Lookup.getDefault().lookupAll(factory.getDataServiceClass()));
        this.factory = factory;
        this.machineDataHolder = machineDataHolder;
    }

    @Override
    public boolean isValid() {
        return true;
    }
    
    @Override
    public void validate() throws WizardValidationException{
        if(!isPanelValid()){
            super.validatePanel();
            throw new WizardValidationException(overviewDecorator, Bundle.MachineWizardStartPanel_invalid_input_message(), Bundle.MachineWizardStartPanel_invalid_input_message());
        }
    }

    @Override
    public void addChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.add(l);
        }
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.remove(l);
        }
    }

    protected final void fireChangeEvent() {
        Set<ChangeListener> ls;
        synchronized (listeners) {
            ls = new HashSet<>(listeners);
        }
        ChangeEvent ev = new ChangeEvent(this);
        for (ChangeListener l : ls) {
            l.stateChanged(ev);
        }
    }

    @Override
    public void readSettings(WizardDescriptor wiz) {
    }

    @Override
    public void storeSettings(WizardDescriptor wiz) {
        wiz.putProperty(MachineWizardIterator.PROP_MACHINE, machineDataHolder);
        wiz.putProperty(MachineWizardIterator.PROP_MODEL, getSelectedModel());
    }

    @Override
    public MachineProvider getMachineProvider() {
        return Lookup.getDefault().lookup(factory.getMachineProviderClass());
    }

    @Override
    public Project getProject() {
        return factory.getProject();
    }

    @Override
    public void fieldsUpdated() {
        fireChangeEvent();
        overviewDecorator.setTitle(getOverviewPanel().getNameField().getText());
    }

    @Override
    public boolean isFinishPanel() {
        return isPanelValid();
    }

    @Override
    public Component getComponent() {
        if (overviewDecorator == null) {
            MachineOverviewPanel overviewPanel = getOverviewPanel();
            overviewDecorator = new ASBodyPanel(overviewPanel);
            overviewDecorator.setName(Bundle.MachineWizardStartPanel_general_properties_step_name());
            overviewDecorator.setIcon(ImageUtilities.loadImageIcon("nl/cloudfarming/client/fleet/vehicle/steering_wheel_332.png", true));
        }
        return overviewDecorator;
    }

    @Override
    public HelpCtx getHelp() {
        return null;
    }
}
